.grid.svelte-we6cl6{display:grid;width:100%}.grid-item.svelte-we6cl6{background:#fff;border:1px solid #e5e7eb;border-radius:.75rem;padding:.75rem}.grid-title.svelte-we6cl6{font-size:.875rem;font-weight:600;margin-bottom:.5rem}.grid-item.equal.svelte-we6cl6{display:flex;flex-direction:column}
