.callout.svelte-y8yhe6{border-radius:.5rem;padding:.75rem 1rem;border:1px solid transparent}.callout-header.svelte-y8yhe6{display:flex;justify-content:space-between;align-items:center;font-weight:600;cursor:default;background:transparent;border:none;width:100%;text-align:left;padding:0;font-size:inherit;color:inherit}button.callout-header.svelte-y8yhe6{cursor:pointer}.callout-body.svelte-y8yhe6{margin-top:.5rem}.callout.info.svelte-y8yhe6{background:#eff6ff;border-color:#bfdbfe}.callout.warning.svelte-y8yhe6{background:#fef9c3;border-color:#fde047}.callout.error.svelte-y8yhe6{background:#fee2e2;border-color:#fecaca}.callout.success.svelte-y8yhe6{background:#dcfce7;border-color:#86efac}.callout.tip.svelte-y8yhe6{background:#ecfccb;border-color:#bef264}.callout.note.svelte-y8yhe6{background:#f8fafc;border-color:#cbd5f5}
