.accordion.svelte-1b7h7k8{padding:.75rem 1rem}details.svelte-1b7h7k8{border:1px solid #e5e7eb;border-radius:.5rem;padding:.5rem;margin-bottom:.5rem;background:#fff}summary.svelte-1b7h7k8{cursor:pointer;font-weight:600}.panel.svelte-1b7h7k8{margin-top:.75rem}
