import{s as a,c as s,a as e,C as t}from"./chunk-B4BG7PRW-DuWDvURh.js";import{_ as i}from"./Mermaid-zTl0VhAq.js";import"./chunk-FMBD7UC4-UFu3HCi_.js";import"./chunk-55IACEB6-pxoiI4WT.js";import"./chunk-QN33PNHL-BYy4nciv.js";import"./index-DW7rebFp.js";import"./_commonjsHelpers-CqkleIqs.js";import"./purify.es-HRjpPm7y.js";var f={parser:e,get db(){return new t},renderer:s,styles:a,init:i(r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute},"init")};export{f as diagram};
