import{s as r,b as e,a,S as s}from"./chunk-DI55MBZ5-BnpuKvnT.js";import{_ as i}from"./Mermaid-zTl0VhAq.js";import"./chunk-55IACEB6-pxoiI4WT.js";import"./chunk-QN33PNHL-BYy4nciv.js";import"./index-DW7rebFp.js";import"./_commonjsHelpers-CqkleIqs.js";import"./purify.es-HRjpPm7y.js";var n={parser:a,get db(){return new s(2)},renderer:e,styles:r,init:i(t=>{t.state||(t.state={}),t.state.arrowMarkerAbsolute=t.arrowMarkerAbsolute},"init")};export{n as diagram};
