.confirm.svelte-1dszvfa{padding:1.25rem;border-radius:.75rem;border:1px solid #e5e7eb;background:#fff}.confirm.warning.svelte-1dszvfa{border-color:#facc15}.confirm.danger.svelte-1dszvfa{border-color:#f87171}.confirm.success.svelte-1dszvfa{border-color:#4ade80}h3.svelte-1dszvfa{margin:0;font-size:1.1rem}.confirm-body.svelte-1dszvfa{margin-top:.75rem}.confirm-actions.svelte-1dszvfa{display:flex;justify-content:flex-end;gap:.75rem;margin-top:1rem}.btn-cancel.svelte-1dszvfa{border:1px solid #d1d5db;background:#fff;padding:.4rem .75rem;border-radius:.375rem;cursor:pointer}.btn-confirm.svelte-1dszvfa{border:none;background:#2563eb;color:#fff;padding:.4rem .75rem;border-radius:.375rem;cursor:pointer}
