.metric.svelte-314kjy{padding:1rem;border-radius:.75rem;border:1px solid #e5e7eb;background:#fff}.metric-header.svelte-314kjy{display:flex;justify-content:space-between;align-items:center;font-size:.75rem;color:#64748b;text-transform:uppercase;letter-spacing:.04em}.metric-value.svelte-314kjy{font-size:1.75rem;font-weight:600;margin-top:.5rem}.metric-change.svelte-314kjy{margin-top:.25rem;font-size:.75rem}.metric-change.positive.svelte-314kjy{color:#16a34a}.metric-change.negative.svelte-314kjy{color:#dc2626}.metric-sparkline.svelte-314kjy{display:flex;gap:2px;align-items:flex-end;margin-top:.5rem}.spark.svelte-314kjy{width:6px;background:#cbd5f5;border-radius:2px}
