import{p as jt,a as C,f as s,i,e as o,h as l,g as t,c as xt,j as F,t as h,F as nt,n as r,m as u,D as K,A as N,o as G,G as it,r as kt,k as wt}from"./index-DW7rebFp.js";import O from"./Markdown-Ds_jAzU-.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var Ct=s('<h1 class="svelte-djhbyc"> </h1>'),Rt=s('<p class="subtitle svelte-djhbyc"> </p>'),At=s("<span> </span>"),Bt=s("<span> </span>"),Dt=s("<span> </span>"),Et=s('<div class="meta svelte-djhbyc"><!> <!> <!></div>'),Ft=s('<li class="svelte-djhbyc"><a class="svelte-djhbyc"> </a></li>'),Gt=s('<nav class="toc svelte-djhbyc"><h2 class="svelte-djhbyc">Contents</h2> <ul class="svelte-djhbyc"></ul></nav>'),Lt=s('<h2 class="svelte-djhbyc"> </h2>'),Mt=s('<figcaption class="svelte-djhbyc"> </figcaption>'),Tt=s('<figure class="component-figure svelte-djhbyc"><!> <!></figure>'),qt=s('<h3 class="svelte-djhbyc"> </h3>'),zt=s('<figcaption class="svelte-djhbyc"> </figcaption>'),Ht=s('<figure class="component-figure svelte-djhbyc"><!> <!></figure>'),It=s('<section class="subsection svelte-djhbyc"><!> <!> <!></section>'),Jt=s('<section class="svelte-djhbyc"><!> <!> <!> <!></section>'),Kt=s('<footer class="report-footer svelte-djhbyc"><!></footer>'),Nt=s('<article class="artifact-report svelte-djhbyc"><header class="report-header svelte-djhbyc"><!> <!> <!></header> <!> <div class="report-body svelte-djhbyc"></div> <!></article>');function Vt(ct,j){jt(j,!0);let P=C(j,"title",3,""),Q=C(j,"subtitle",3,""),g=C(j,"metadata",19,()=>({})),S=C(j,"toc",3,!1),U=C(j,"sections",19,()=>[]),V=C(j,"footer",3,"");function W(a){return a.toLowerCase().replace(/\s+/g,"-").replace(/[^\w-]/g,"")}function X(a,e=0){const v=[];for(const d of a)d.title&&v.push({title:d.title,id:d.id||W(d.title),level:e}),d.subsections&&v.push(...X(d.subsections,e+1));return v}const Y=xt(()=>S()?X(U()):[]);var Z=Nt(),$=o(Z),tt=o($);{var lt=a=>{var e=Ct(),v=o(e);h(()=>u(v,P())),r(a,e)};i(tt,a=>{P()&&a(lt)})}var et=l(tt,2);{var dt=a=>{var e=Rt(),v=o(e);h(()=>u(v,Q())),r(a,e)};i(et,a=>{Q()&&a(dt)})}var _t=l(et,2);{var ft=a=>{var e=Et(),v=o(e);{var d=c=>{var p=At(),k=o(p);h(()=>u(k,`By ${g().author??""}`)),r(c,p)};i(v,c=>{g().author&&c(d)})}var b=l(v,2);{var m=c=>{var p=Bt(),k=o(p);h(()=>u(k,g().date)),r(c,p)};i(b,c=>{g().date&&c(m)})}var w=l(b,2);{var x=c=>{var p=Dt(),k=o(p);h(()=>u(k,`v${g().version??""}`)),r(c,p)};i(w,c=>{g().version&&c(x)})}r(a,e)};i(_t,a=>{(g().author||g().date)&&a(ft)})}var at=l($,2);{var ht=a=>{var e=Gt(),v=l(o(e),2);F(v,21,()=>t(Y),G,(d,b)=>{var m=Ft();let w;var x=o(m),c=o(x);h(()=>{w=wt(m,"",w,{"margin-left":`${t(b).level*1}rem`}),nt(x,"href",`#${t(b).id}`),u(c,t(b).title)}),r(d,m)}),r(a,e)};i(at,a=>{S()&&t(Y).length>0&&a(ht)})}var rt=l(at,2);F(rt,21,U,G,(a,e)=>{var v=Jt(),d=o(v);{var b=n=>{var y=Lt(),R=o(y);h(()=>u(R,t(e).title)),r(n,y)};i(d,n=>{t(e).title&&n(b)})}var m=l(d,2);{var w=n=>{O(n,{get content(){return t(e).content}})};i(m,n=>{t(e).content&&n(w)})}var x=l(m,2);{var c=n=>{var y=K(),R=N(y);F(R,17,()=>t(e).components,G,(q,_)=>{var A=Tt(),B=o(A);it(B,{get component(){return t(_).component},get props(){return t(_).props}});var z=l(B,2);{var L=D=>{var M=Mt(),H=o(M);h(()=>u(H,t(_).caption)),r(D,M)};i(z,D=>{t(_).caption&&D(L)})}r(q,A)}),r(n,y)};i(x,n=>{t(e).components&&n(c)})}var p=l(x,2);{var k=n=>{var y=K(),R=N(y);F(R,17,()=>t(e).subsections,G,(q,_)=>{var A=It(),B=o(A);{var z=f=>{var E=qt(),I=o(E);h(()=>u(I,t(_).title)),r(f,E)};i(B,f=>{t(_).title&&f(z)})}var L=l(B,2);{var D=f=>{O(f,{get content(){return t(_).content}})};i(L,f=>{t(_).content&&f(D)})}var M=l(L,2);{var H=f=>{var E=K(),I=N(E);F(I,17,()=>t(_).components,G,(gt,T)=>{var ot=Ht(),vt=o(ot);it(vt,{get component(){return t(T).component},get props(){return t(T).props}});var bt=l(vt,2);{var mt=J=>{var st=zt(),yt=o(st);h(()=>u(yt,t(T).caption)),r(J,st)};i(bt,J=>{t(T).caption&&J(mt)})}r(gt,ot)}),r(f,E)};i(M,f=>{t(_).components&&f(H)})}r(q,A)}),r(n,y)};i(p,n=>{t(e).subsections&&n(k)})}h(n=>nt(v,"id",n),[()=>t(e).id||(t(e).title?W(t(e).title):void 0)]),r(a,v)});var pt=l(rt,2);{var ut=a=>{var e=Kt(),v=o(e);O(v,{get content(){return V()}}),r(a,e)};i(pt,a=>{V()&&a(ut)})}r(ct,Z),kt()}export{Vt as default};
