import{a as s,f as d,j as k,i as G,g as a,e as p,h as H,G as S,t as o,l as j,k as g,n,m as y,o as C}from"./index-DW7rebFp.js";var R=d('<div class="grid-title svelte-we6cl6"> </div>'),z=d("<div><!> <!></div>"),A=d('<div class="grid svelte-we6cl6"></div>');function D(m,t){let u=s(t,"columns",3,2),_=s(t,"gap",3,"1rem"),f=s(t,"items",19,()=>[]),h=s(t,"equalHeight",3,!0);var l=A();k(l,21,f,C,(w,e)=>{var r=z(),v=p(r);{var x=i=>{var c=R(),b=p(c);o(()=>y(b,a(e).title)),n(i,c)};G(v,i=>{a(e).title&&i(x)})}var q=H(v,2);S(q,{get component(){return a(e).component},get props(){return a(e).props}}),o(()=>{j(r,1,`grid-item ${h()?"equal":""}`,"svelte-we6cl6"),g(r,`grid-column: span ${a(e).colSpan??1}; grid-row: span ${a(e).rowSpan??1};`)}),n(w,r)}),o(()=>g(l,`grid-template-columns: repeat(${u()}, minmax(0, 1fr)); gap: ${_()};`)),n(m,l)}export{D as default};
