.markdown.svelte-yueq7f{padding:1rem;font-size:.95rem;line-height:1.6}.markdown.svelte-yueq7f h1,.markdown.svelte-yueq7f h2,.markdown.svelte-yueq7f h3,.markdown.svelte-yueq7f h4,.markdown.svelte-yueq7f h5,.markdown.svelte-yueq7f h6{margin-top:1.5em;margin-bottom:.5em;font-weight:600}.markdown.svelte-yueq7f h1:first-child,.markdown.svelte-yueq7f h2:first-child,.markdown.svelte-yueq7f h3:first-child{margin-top:0}.markdown.svelte-yueq7f p{margin:.75em 0}.markdown.svelte-yueq7f ul,.markdown.svelte-yueq7f ol{margin:.75em 0;padding-left:1.75em}.markdown.svelte-yueq7f li{margin:.35em 0}.markdown.svelte-yueq7f li>ul,.markdown.svelte-yueq7f li>ol{margin:.25em 0}.markdown.svelte-yueq7f code{font-family:var(--font-mono, ui-monospace);background:#f1f5f9;padding:.1rem .25rem;border-radius:.25rem}.markdown.svelte-yueq7f pre{background:#0f172a;color:#e2e8f0;padding:.75rem;border-radius:.5rem;overflow-x:auto;margin:1em 0}.markdown.svelte-yueq7f pre code{background:transparent;padding:0}.markdown.svelte-yueq7f blockquote{margin:1em 0;padding:.5em 1em;border-left:3px solid #cbd5e1;background:#f8fafc;color:#475569}.markdown.svelte-yueq7f table{border-collapse:collapse;margin:1em 0;width:100%}.markdown.svelte-yueq7f th,.markdown.svelte-yueq7f td{border:1px solid #e2e8f0;padding:.5em .75em;text-align:left}.markdown.svelte-yueq7f th{background:#f8fafc;font-weight:600}.markdown.svelte-yueq7f hr{border:none;border-top:1px solid #e2e8f0;margin:1.5em 0}
