.tabs.svelte-1f2pv7a{padding:1rem}.tab-list.svelte-1f2pv7a{display:flex;gap:.5rem;flex-wrap:wrap}.tab.svelte-1f2pv7a{padding:.4rem .75rem;border-radius:.5rem;border:1px solid #e5e7eb;background:#fff;cursor:pointer;font-size:.8rem}.tab.active.svelte-1f2pv7a{background:#2563eb;color:#fff;border-color:#2563eb}.tab-panel.svelte-1f2pv7a{margin-top:1rem}.tabs.pills.svelte-1f2pv7a .tab:where(.svelte-1f2pv7a){border-radius:999px}.tabs.enclosed.svelte-1f2pv7a{border:1px solid #e5e7eb;border-radius:.75rem}
