import{p as B,a as t,f as h,h as n,e,i as E,t as F,l as G,m as r,n as _,r as I,x as J}from"./index-DW7rebFp.js";import b from"./Markdown-Ds_jAzU-.js";import"./purify.es-HRjpPm7y.js";import"./katex-XbL3y5x-.js";/* empty css                  */var K=h("<details><summary>Details</summary> <!></details>"),N=h('<div><div class="confirm-header"><h3 class="svelte-1dszvfa"> </h3></div> <div class="confirm-body svelte-1dszvfa"><!> <!></div> <div class="confirm-actions svelte-1dszvfa"><button class="btn-cancel svelte-1dszvfa"> </button> <button class="btn-confirm svelte-1dszvfa"> </button></div></div>');function U(g,a){B(a,!0);let x=t(a,"title",3,"Confirm"),z=t(a,"message",3,""),C=t(a,"confirmLabel",3,"Confirm"),k=t(a,"cancelLabel",3,"Cancel"),L=t(a,"variant",3,"info"),o=t(a,"details",3,void 0),v=t(a,"onResult",3,void 0);function w(){v()?.({confirmed:!0})}function y(){v()?.({confirmed:!1})}var i=N(),d=e(i),H=e(d),R=e(H),c=n(d,2),f=e(c);b(f,{get content(){return z()},allowHtml:!1});var p=n(f,2);{var D=s=>{var u=K(),A=n(e(u),2);b(A,{get content(){return o()},allowHtml:!1}),_(s,u)};E(p,s=>{o()&&s(D)})}var M=n(c,2),l=e(M);l.__click=y;var j=e(l),m=n(l,2);m.__click=w;var q=e(m);F(()=>{G(i,1,`confirm ${L()}`,"svelte-1dszvfa"),r(R,x()),r(j,k()),r(q,C())}),_(g,i),I()}J(["click"]);export{U as default};
